use crate::patterns::{Pattern, PixelColor};
use crate::Strip;
use serde::{Serialize, Deserialize};

/// # scanner pattern
/// color one pixel with a color and leave a train of this color fading to black
///
/// # note
///
/// background_color work only for color not set by scanner pattern
#[derive(Serialize, Deserialize, Debug, Eq, Hash, PartialEq, Copy, Clone)]
pub struct Scanner<const N: usize> {
    #[serde(default)]
    pub(crate) current_iteration: usize,
    pub(crate) color: PixelColor,
    pub(crate) background_color: Option<PixelColor>,
    #[serde(default)]
    pub(crate) max_iteration: Option<usize>,
}

impl<const N:usize> Pattern for Scanner<N> {
    type Strip = Strip<N>;

    fn init(&mut self) -> Option<Self::Strip> {
        if let Some(color) = self.background_color {
            Some(Strip::<N>::new(color))
        } else {
            None
        }
    }

    fn is_last_iteration(&self) -> bool {
        match self.max_iteration {
            None => { false }
            Some(max_iter) => {
                if self.current_iteration >= max_iter {
                    true
                } else {
                    false
                }
            }
        }
    }
}

impl<const N: usize> Iterator for Scanner<N> {
    type Item = Strip<N>;

    fn next(&mut self) -> Option<Self::Item> {
        if self.is_last_iteration() {
            return None;
        }

        let mut strip : Strip<N>;

        if let Some(color) = self.background_color {
            strip = Strip::new(color);
        } else {
            strip = Strip::default();
        }

        let current_led = self.current_iteration % N;
        let min_led;
        if current_led < 8 {
            min_led = 0;
        } else {
            min_led = current_led - 8;
        }
        let mut c = self.color;
        for i in min_led..current_led {
            strip[i] = Some(c);
            c = c / 2;
        }
        self.current_iteration += 1;
        Some(strip)
    }
}